# R code illustrating sensitivity analysis by perturbation 
# from Ellner and Rees, Am Nat (2005 or thereabouts). 
# See Appendix A section 5 of the paper for more details.
# The code can be copied and pasted directly into R, or sourced.
#

#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*
#*#* Chunk 0: settings

#Set matrix size and convergence tolerance 
bigM=250	#make bigger to truncate
tol=1.e-10 

# Function to do an image plot of a matrix in the usual orientation, A(1,1) at top left  
matrix.image=function(x,y,A,col=topo.colors(100),...) {
	nx=length(x); ny=length(y); 
	x1=c(1.5*x[1]-0.5*x[2],1.5*x[nx]-0.5*x[nx-1]); 
	y1=c(1.5*y[1]-0.5*y[2],1.5*y[ny]-0.5*y[ny-1]); 
	image(list(x=x,y=y,z=t(A)),xlim=x1,ylim=rev(y1),col=col,bty="u",...);  
}

#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*
#*#* This first chunk plots the data and the regression lines used to fill the IPM

#*#* Size classes by 100um increments
xv <- seq(150, 950, 100)


#*#* mortality rates from Bijma & Hemleben 1994 per size class
yv <- c(0.61,0.36,0.17,0.20,0.37,0.44,0.42,0.76,1.00)

#*#* survival = 1 - mortality. Ignore final class because the 0 causes problems in model fit
h  <- 1-yv

#*#* need to adjust the first few because survival across those size classes takes multiple time steps
h[1] <- h[1]^(1/3)
h[2] <- h[2]^(1/2) #count no. of size transitions in gg below
xx <- log(xv)

#*#* reproduction rates from Bijma & Hemleben 1994 Figure 4
rr <- c(0,0,3,21,55,73,56,72,84,83,73,82)/100
#*#* reproduction size classes by 50um incremements
rx <- log(seq(125,675,50))

#*#* Figure 2
pdf(paste(Sys.Date(), "_vitalrates.pdf", sep=""), width = 2.7, height = 5.5)

# Panel a
par(mfcol=c(3,1),cex.axis=1.35,cex.lab=1.35,bty="o", mar=c(4, 4.5, 1, .2), mgp=c(2.5, .7, 0)); 
plot(xx, h, las=1, xlab="ln(Size)", ylab="Survival", ylim=range(0,1), xlim=range(4,7),  
  cex=1.5, pch=16, col=rgb(0,0,0,.7))
#*#* a is the asymptote level
#*#* b controls the steepness of the increase
#*#* c controls the point on x-axis at which increase starts (inflexion point, actuall)
fit <- nls(h ~ a1*xx + a2*exp(a3*xx), start=list(a1=0.1582, a2=-3.228e-07, a3=2))
lines(seq(4.8, 7, .1), predict(fit, newdata=data.frame(xx=seq(4.8, 7, .1))))
text(4.07, 0.92, 'A', cex=1.5)

# calculate adjusted R squared
pred <- predict(fit)
n <- length(pred)
res <- resid(fit)
w <- weights(fit) 
if(is.null(w)) w <- rep(1, n)
rss <- sum(w * res ^ 2)
resp <- pred + res
center <- weighted.mean(resp, w)
r.df <- summary(fit)$df[2]
int.df <- 1
tss <- sum(w * (resp - center)^2)
r.sq <- 1 - rss/tss
adj.r.sq <- 1 - (1 - r.sq) * (n - int.df) / r.df
out <- list(pseudo.R.squared = r.sq,
            adj.R.squared = adj.r.sq)

# Panel b
plot(rx, rr, las=1, xlab="ln(Size)", ylab="Prob. reproduction", ylim=range(0,1), xlim=range(4,7), 
  cex=1.5, pch=16, col=rgb(0,0,0,.7))
text(4.07, 0.92, 'B', cex=1.5)
#*#* a is the asymptote level
#*#* b controls the steepness of the increase
#*#* c controls the point on x-axis at which increase starts (inflexion point, actuall)
fit <- nls(rr ~ a/(1+exp(b*(rx-c))), start=list(a=0.76687, b=-9.63430, c=5.70592))
lines(seq(4.7, 6.7, .1), predict(fit, newdata=data.frame(rx=seq(4.7, 6.7, .1))))
#*#* or lines(0:700, predict(fit, newdata=data.frame(rx=0:700)), lwd=2)

#*# calculate adjusted R squared
pred <- predict(fit)
n <- length(pred)
res <- resid(fit)
w <- weights(fit) 
if(is.null(w)) w <- rep(1, n)
rss <- sum(w * res ^ 2)
resp <- pred + res
center <- weighted.mean(resp, w)
r.df <- summary(fit)$df[2]
int.df <- 1
tss <- sum(w * (resp - center)^2)
r.sq <- 1 - rss/tss
adj.r.sq <- 1 - (1 - r.sq) * (n - int.df) / r.df
out <- list(pseudo.R.squared = r.sq,
            adj.R.squared = adj.r.sq)

coef(fit)
sig <- function(p, x) p[1]/(1+exp(p[2]*(x-p[3])))
gg <- c(20, 25, 30, 40, 45, 50, 55, 65, 70, 80, 90, 105, 130, 160, 220, 340, 500, 710)
sznxt <- gg[-1]
sz <- gg[-18]
#*#* the above is the plot in Schmidt et al.

logsznxt <- log(sznxt)
logsz <- log(sz)

# Panel c
plot(logsznxt ~ logsz, xlab="ln(Size) at stage t", ylab="ln(Size) at stage (t+1)", 
  cex=1.5, pch=16, col=rgb(0,0,0,.7))
text(3.07, 6.3, 'C', cex=1.5)
#*#* force through origin using -1
mg <- lm(logsznxt ~ logsz - 1)
summary(mg)

msz <- logsz - mean(logsz)
mszn <- logsznxt - mean(logsznxt)
mg <- lm(mszn ~ msz - 1)
summary(mg)

pg <- predict(mg, newdata=data.frame(msz=seq(3, 7, 0.1)))
lines(seq(3, 7, 0.1), pg)
dev.off()

#============================================================================# 
#*#* Parameters
#============================================================================# 
pv <- c(s1=0.1478, s2=-5.126e-09, s3=2.784,		#survival parameters
		f1=0.77, f2=-9.63, f3=5.71,
		g1=1.0768, g2=0.0238, g3=0.185,
		ggs=.01585, offmu=2.6, offsg=0.1)

#============================================================================# 
#*#*  Define the kernel K(y,x) and iteration matrix, for x=size in range [0,5]
#*#* these use logged sizes, and we do at least get a decent reproductive value
#*#* now need to sort the survivorship
#============================================================================# 
# Probability of survival: concave-up (y=ax^b) regression on x=log(size) and age
sx<-function(x) 
  {
  u <- pv['s1']*x + pv['s2']*exp(pv['s3']*x)
  u[u<0] <- 0
  return(u)
  }
#p[1]*exp(-p[2]*x) + p[3] + p[4]*exp(p[5]*x)

# Probability of reproduction, logistic regression on size and age
fx<-function(x) {pv['f1']/(1+exp(pv['f2']*(x-pv['f3'])))}

# Growth function, normal distribution with mean (mux) and variance (sigmax2) depending on size
# x=size now, y= size next year. 
gyx<-function(y,x)
  {
  mux<-pv['g1']*x			#intercept is zero, so drop params[8] bit
  sigmax2<-pv['g2']*exp(2*(pv['g3']*mux))	#guess
  sigmax<-sqrt(sigmax2)

  fac1<-sqrt(2*pi)*sigmax
  fac2<-((y-mux)^2)/(2*sigmax2)
  return(exp(-fac2)/fac1)
  }

###### reproductive output
fyx=function(y,x) {((exp(x)*pv['ggs'])^3) * fx(x) * sx(x) * dnorm(y,mean=pv['offmu'],sd=pv['offsg'])}
pyx=function(y,x) {sx(x) * (1-fx(x)) * gyx(y,x)}
kyx=function(y,x) {pyx(y,x)+fyx(y,x)}

# Compute meshpoints iteration matrix KD 
# Note the use of outer() to compute kernel values at all meshpoints in one statement.  
h=5/bigM
y=2+(h/2)*((0:(bigM-1))+(1:bigM));  
K=outer(y,y,kyx)
KD=h*K;  


#============================================================================# 
# Sensitivity analysis I: from the formulas 
#============================================================================# 
# Get lamda,v,w from the iteration matrix, and plot 
lam.eigen=Re(eigen(KD)$values[1]); 
print(lam.eigen)
w.eigen=Re(eigen(KD)$vectors[,1])
w.eigen=w.eigen/sum(w.eigen)
v.eigen=Re(eigen(t(KD))$vectors[,1])
v.eigen=v.eigen/v.eigen[1]

#*#* Figure 3
png(paste(Sys.Date(), "_kernel_SSDRV.png", sep=""), 3.5, 6.8,  unit="in", res=300)
par(oma=c(2,0,1.5,.5), mar=c(.5, 3, 1, .2), tcl=0.3, mgp=c(1.5, 0.05, 0), mfrow=c(2,1), cex.axis=.9)
matrix.image(y,y, log(K+0.0001), axes=F, col=grey(100:0/100), las=1)
axis(3, at=c(2,3,4,5,6,7), tck=0)
axis(2, at=c(2,3,4,5,6,7), tck=0, las=1, mgp=c(1.5, 0.5, 0))
mtext("ln(size) at stage t", side=3, line=1)
mtext("ln(size) at stage (t+1)", side=2, las=0, line=2)
contour(y,y,log(t(K)+0.0001),add=TRUE,cex=3, col="yellow");
abline(h=0); abline(v=0);
text(2.2, 2.2, 'A', cex=1.1)
box(bty='o')


plot(y,w.eigen,type="l", xlab='', ylab='', bty='l', lwd=2, las=1)
mtext('ln(size)', side=1, line=1.5)
mtext('Eigenvector', side=2, line=2)
text(6.2,0.032,bquote(lambda [1] == .(round(lam.eigen, 3))))
lines(y,v.eigen*mean(w.eigen)/mean(v.eigen),lty=2, lwd=2);
text(2.05, 0.0535, 'B', cex=1.1)
legend(5, 0.055, legend=c('SSD', 'RV'), lty=c(1, 2), lwd=1.2, bty='n', seg.len=1.2)
dev.off()



#============================================================================# 
# Sensitivity analysis I: from the formulas 
#============================================================================# 
# Get lamda,v,w from the iteration matrix, and plot 
lam.eigen=Re(eigen(KD)$values[1]); 
w.eigen=Re(eigen(KD)$vectors[,1])
w.eigen=w.eigen/sum(w.eigen)
v.eigen=Re(eigen(t(KD))$vectors[,1])
v.eigen=v.eigen/v.eigen[1]

# Compute sensitivity and elasticity using sensitivity formulas 
v.dot.w= h*sum(v.eigen*w.eigen) # note <v,w> is an integral, done here by midpoint rule.  
sens.eigen=outer(v.eigen,w.eigen)/v.dot.w   
elas.eigen=K*sens.eigen/lam.eigen 

#============================================================================# 
# Plot the results 
#============================================================================# 
#matrix.image(y,y,elas.eigen,xlab="ln(Size) at t",ylab="ln(Size) at t+1",col=topo.colors(100)); 
#title("Elasticity"); 
#contour(y,y,t(elas.eigen),add=T,cex=3);
#abline(h=0); abline(v=0); 

#============================================================================# 
# numerical perturbation to get lower-level elasticities.
#============================================================================# 
#IPM pack uses delta = 1e-04

delta = 1e-04
lp <- length(pv)
lle <- numeric(lp)
pvn <- pv

for(k in 1:lp)
  {
  pv <- pvn
  pv[k] <- pv[k]*(1+delta)
  sx<-function(x) 
  {
  u <- pv['s1']*x + pv['s2']*exp(pv['s3']*x)
  u[u<0] <- 0
  return(u)
  }


# Probability of flowering, logistic regression on size and age
fx<-function(x) {pv['f1']/(1+exp(pv['f2']*(x-pv['f3'])))}
# Growth function, normal distribution with mean (mux) and variance (sigmax2) depending on size
# x=size now, y= size next year. 
gyx<-function(y,x)
  {
  mux<-pv['g1']*x			#intercept is zero, so drop params[8] bit
  sigmax2<-pv['g2']*exp(2*(pv['g3']*mux))	#guess
  sigmax<-sqrt(sigmax2)

  fac1<-sqrt(2*pi)*sigmax
  fac2<-((y-mux)^2)/(2*sigmax2)
  return(exp(-fac2)/fac1)
  }

###### reproductive output
fyx=function(y,x) {((exp(x)*pv['ggs'])^3) * fx(x) * sx(x) * dnorm(y,mean=pv['offmu'],sd=pv['offsg'])}
pyx=function(y,x) {sx(x) * (1-fx(x)) * gyx(y,x)}
kyx=function(y,x) {pyx(y,x)+fyx(y,x)}

# Compute meshpoints iteration matrix KD 
# Note the use of outer() to compute kernel values at all meshpoints in one statement.  
h=5/bigM
y=2+(h/2)*((0:(bigM-1))+(1:bigM));  
K=outer(y,y,kyx)
KD=h*K;  

  lamp.eigen=Re(eigen(KD)$values[1]); 
  lle[k] <- lamp.eigen - lam.eigen
  }


llec <- abs(lle)
llecm <- matrix(llec*1000, nrow=4, byrow=TRUE)
cl <- rep(c(2,4, "seagreen", "orange"), each=3)



#*#* Figure 4
png(paste(Sys.Date(), "_llelasticities_talks.png", sep=""),  6.8, 2.9, unit="in", res=300)
par(mar=c(3, 3, .5, .2), las=1, tcl=0.3, mgp=c(1.5, 0.25, 0))
bp <- barplot(t(llecm), beside=TRUE, ylim=range(0, 0.57), 
  names.arg=c("Survival", "Fertility", "Growth", "Recruitment"), 
  col=cl,  border=cl, space=c(0.1, 1),
  xlab="Vital rate", ylab=expression(paste("Elasticity to ", lambda [1])), cex.lab=1.5)
c <- vector(mode='character', length=12)
c[1] <- expression(paste(sigma[1]))
text(bp, t(llecm)+0.03, 
     c(expression(paste(italic('s'[1]))), expression(paste(italic('s'[2]))), expression(paste(italic('s'[3]))),
       expression(paste(italic('f'[1]))), expression(paste(italic('f'[2]))), expression(paste(italic('f'[3]))), 
       expression(paste(italic('g'[1]))), expression(paste(italic('g'[2]))), expression(paste(italic('g'[3]))), 
       expression(paste(zeta)), expression(paste(mu['off'])), expression(paste(sigma['off']))))
dev.off()

rowSums(llecm)
0.4833788/0.2822995
#1] 1.712291
0.4833788/0.0603574
#[1] 8.008609
0.4833788/0.1318610
#[1] 3.665821